# -*- perl -*-
#
# Copyright (C) 1996 Ken MacLeod
# See the file COPYING for distribution terms.
#

package Text::GDO::Node;

sub new {
    my ($type, %value) = @_;

    my ($self) = \%value;

    bless $self, $type;
}

#
# return a list of all nodes beginning with the root node and ending
# with $self.  Do this by creating a list of all the parents, using
# `unshift' to push them on ``most recent (rootward) first''
#
sub rootpath {
    my ($self) = @_;
    my (@path, $node);

    $node = $self;
    do {
	unshift (@path, $node);
    } while ($node = $node->{'parent'});

    return @path;
}

#
# return the root node
#
sub rootnode {
    my ($self) = @_;
    my ($next_node, $node);

    $next_node = $self;
    do {
	$node = $next_node;
    } while ($next_node = $node->{'parent'});

    return $node;
}

sub reference {
    my ($self, $id) = @_;

    # if we've already located it, return it again
    if (defined $self->{'referables'}{$id}) {
	return ($self->{'referables'}{$id}{'object'});
    }

    # if we haven't already located it, find it with a breadth-first
    # search.  A bread-first search will hit the higher nodes first,
    # the more likely places to find objects marked with IDs.

    # we will cache every ID we find, so we create a search context so
    # that every time we start another search we'll start from where
    # we left off last time.

    if (!defined $self->{'reference_context'}) {
	# our starting point is ``here''
	$self->{'reference_context'} = [[$self, 0]];
    }

    # XXX hmm, this idea suddenly got a lot more difficult to
    # implement.  In the SGML tree, or the DocObject tree?  in the
    # `content' or a DocObject subclass method?  For now, `saveAttr'
    # in each spec will satisfy the above cache.

    return undef;
}

1;
